using Xunit;
using Moq;


namespace MoqCountExecutionTimes.Tests
{
    public class UnitTest
    {

        [Fact]
        public void Check_Times_Called_With_Verify()
        {
            // Arrange
            Mock<IMathService> mathService = new Mock<IMathService>();
            mathService.Setup(ms => ms.AddNumbers(1, 2)).Returns(3);
            
            FooService fooService = new FooService(mathService.Object);
            
            // Act 
            string result1 = fooService.AddMessageToAnswer("hello", 1, 2);

            // Assert
            Assert.Equal("hello 3", result1);
            mathService.Verify(ms => ms.AddNumbers(1, 2), Times.Exactly(1));
        }

        [Fact]
        public void Check_Times_Called_With_Callback()
        {
            // Arrange
            int timesCalled = 0;

            Mock<IMathService> mathService = new Mock<IMathService>();
            mathService.Setup(ms => ms.AddNumbers(1, 2)).Returns(3).Callback(() => ++timesCalled);
            
            FooService fooService = new FooService(mathService.Object);
            
            // Act 
            string result = fooService.AddMessageToAnswer("hello", 1, 2);

            // Assert
            Assert.Equal("hello 3", result);
            Assert.Equal(1, timesCalled);
        }

        // More examples

        [Fact]
        public void Check_Times_Called_With_Callback_IsAny_In_Arrange()
        {
            // Arrange
            int timesCalled = 0;

            Mock<IMathService> mathService = new Mock<IMathService>();
            mathService.Setup(ms => ms.AddNumbers(It.IsAny<int>(), It.IsAny<int>())).Returns(3).Callback(() => ++timesCalled);
            
            FooService fooService = new FooService(mathService.Object);
            
            // Act 
            string result = fooService.AddMessageToAnswer("hello", 1, 2);

            // Assert
            Assert.Equal("hello 3", result);
            Assert.Equal(1, timesCalled);
        }

        [Fact]
        public void Check_Times_Called_With_Verify_Two_Set_Of_Exact_Params()
        {
            // Arrange
            Mock<IMathService> mathService = new Mock<IMathService>();
            mathService.Setup(ms => ms.AddNumbers(1, 2)).Returns(3);
            mathService.Setup(ms => ms.AddNumbers(3, 4)).Returns(7);
            
            FooService fooService = new FooService(mathService.Object);
            
            // Act 
            string result1 = fooService.AddMessageToAnswer("hello", 1, 2);
            string result2 = fooService.AddMessageToAnswer("goodbye", 3, 4);

            // Assert
            Assert.Equal("hello 3", result1);
            Assert.Equal("goodbye 7", result2);
            mathService.Verify(ms => ms.AddNumbers(1, 2), Times.Exactly(1));
            mathService.Verify(ms => ms.AddNumbers(3, 4), Times.Exactly(1));
        }

        [Fact]
        public void Check_Times_Called_With_Verify_Using_IsAny_In_Assert()
        {
            // Arrange
            Mock<IMathService> mathService = new Mock<IMathService>();
            mathService.Setup(ms => ms.AddNumbers(1, 2)).Returns(3);
            mathService.Setup(ms => ms.AddNumbers(3, 4)).Returns(7);
            
            FooService fooService = new FooService(mathService.Object);
            
            // Act 
            string result1 = fooService.AddMessageToAnswer("hello", 1, 2);
            string result2 = fooService.AddMessageToAnswer("goodbye", 3, 4);

            // Assert
            Assert.Equal("hello 3", result1);
            Assert.Equal("goodbye 7", result2);
            mathService.Verify(ms => ms.AddNumbers(It.IsAny<int>(), It.IsAny<int>()), Times.Exactly(2));
        }

        [Fact]
        public void Check_Times_Called_With_Verify_Using_IsAny_In_Arrange_And_Assert()
        {
            // Arrange
            Mock<IMathService> mathService = new Mock<IMathService>();
            mathService.Setup(ms => ms.AddNumbers(It.IsAny<int>(), It.IsAny<int>())).Returns(3);
            
            FooService fooService = new FooService(mathService.Object);
            
            // Act 
            string result = fooService.AddMessageToAnswer("hello", 1, 2);

            // Assert
            Assert.Equal("hello 3", result);
            mathService.Verify(ms => ms.AddNumbers(It.IsAny<int>(), It.IsAny<int>()), Times.Exactly(1));
        }
    }
}